/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 **********************************************************************/

#ifdef USE_UI_GTK
#ifndef GTK_POVERTY_HEADER
#define GTK_POVERTY_HEADER

#include "../../card/hand.h"
class Player;

namespace GTK_DOKO {
#include <gdk/gdktypes.h>
#if defined(GTK_2_0) || defined(WINDOWS)
#include <gdk/gdkevents.h>
#endif

  class Poverty {
    public:
      // constructor -- set the table
      Poverty(class Table* table);

      // initializes the poverty
      void init();

      // the variables
      class GTK_GUI& gui(); // the gui
      class Table& table(); // the table

      // returns whether the shifting is valid
      bool shifting_valid(Player const& player);

      // 'player' shifts 'cardno' cards
      void shift(Player const& player, unsigned const cardno);
      // ask 'player' whether to accept the poverty
      void ask(Player const& player, unsigned const cardno);
      // the player 'player' has denied the poverty trumps
      void take_denied(Player const& player);
      // all players have denied to take the cards
      void take_denied_by_all();
      // the player 'player' has accepted the poverty trumps
      // and has returned 'cardno' cards with 'trumpno' trumps
      void take_accepted(Player const& player,
			 unsigned const cardno,
			 unsigned const trumpno);
      // 'player' shifts cards
      Hand shift(Player& player);
      // returns whether 'player' accepts the poverty
      bool take_accept(Player& player,
		       unsigned const cardno);
      // the player changes the cards of the poverty
      Hand cards_change(Player& player,
			Hand const& cards_shifted);
      // the poverty player 'player' gets 'cards'
      void cards_get_back(Player& player, Hand const& cards_returned);

      // start and end position of the card / the arrow
      GdkPoint card_pos(Player const& player, unsigned const cardno);
      // outline rectangle of the arrow
      GdkRectangle arrow_outline(Player const& player);

      // draws the poverty elements
      void draw(Player const& player, bool const update = true);

      // a mouse click
      bool mouse_click(GdkEventButton* button_event);
      // removes the card from the hand and adds it in the middle
      void add_card_to_shift(unsigned const pos);

      /// sets the color of the font (from ::settting)
      void arrow_color_set();

    private:
      class Table* const table_p; // the table, this poverty belongs to

      vector<Card> cards; // the cards that are shifted
    public:
      bool shift_cards; // whether to shift the cards
      bool accept_cards; // accept the shifted cards
      bool finished; // the shifting is finished
    private:

      // the GC for the color
      GdkGC* gc;

    private: // not to be used
      Poverty();
      Poverty(Poverty const&);
      Poverty& operator=(Poverty const&);
  }; // class Poverty

} // namspace GTK_DOKO
#endif // #ifndef GTK_POVERTY_HEADER
#endif // #ifdef USE_UI_GTK
