/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTK

#include "XGtkCheckButton.h"

#include "../../../misc/language.h"

namespace GTK_DOKO
{

  /*****************************************************************************
   *
   * XGtkCheckButton::XGtkCheckButton(string const& text)
   *
   * Parameters:	text - text for the button
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Constructor
   *
   ****************************************************************************/
  XGtkCheckButton::XGtkCheckButton(string const& text) :
    XGtkWidget(),
  button_p(gtk_check_button_new_with_label("")) // the button
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkCheckButton::XGtkCheckButton()");

    gtk_object_set_data(GTK_OBJECT(this->button()),
			"text",
			new string(text));

    // set the text of the check widget
    language_update();

    // show all widgets
    gtk_widget_show_all(widget());

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkCheckButton::XGtkCheckButton()");
  } // XGtkCheckButton::XGtkCheckButton()

  /*****************************************************************************
   *
   * XGtkCheckButton::~XGtkCheckButton()
   *
   * Parameters:	-
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Destructor
   *
   ****************************************************************************/
  XGtkCheckButton::~XGtkCheckButton()
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkCheckButton::~XGtkCheckButton()");

    delete (string*)gtk_object_get_data(GTK_OBJECT(this->button()), "text");
    gtk_widget_destroy(this->widget());

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkCheckButton::~XGtkCheckButton()");
  } // XGtkCheckButton::XGtkCheckButton()

  /*****************************************************************************
   *
   * GtkWidget* XGtkCheckButton::widget()
   *
   * Parameters:	-
   *
   * Result:	the widget (container)
   *
   * Version:	0.4.3
   *
   * Description:	Result
   *
   ****************************************************************************/
  GtkWidget*
    XGtkCheckButton::widget()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkCheckButton::widget()");

      DEBUG_ASSERTION((button_p != NULL),
		      "XGtkCheckButton::widget()\n"
		      "  `button_p` == NULL");

      DEBUG_RETURNING(button_p,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkCheckButton::widget()");
    } // GtkWidget* XGtkCheckButton::widget()

  /*****************************************************************************
   *
   * GtkWidget* XGtkCheckButton::button()
   *
   * Parameters:	-
   *
   * Result:	the button
   *
   * Version:	0.4.3
   *
   * Description:	Result
   *
   ****************************************************************************/
  GtkWidget*
    XGtkCheckButton::button()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkCheckButton::button()");

      DEBUG_ASSERTION((this->button_p != NULL),
		      "XGtkCheckButton::button()\n"
		      "  `button_p` == NULL");

      DEBUG_RETURNING(button_p,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkCheckButton::button()");
    } // GtkWidget* XGtkCheckButton::button()

  /*****************************************************************************
   *
   * void XGtkCheckButton::language_update()
   *
   * Parameters:	-
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	updates the language
   *
   ****************************************************************************/
  void
    XGtkCheckButton::language_update()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_UPDATE,
		    "XGtkCheckButton::language_update()");

      // update the text of the label
      gtk_label_set_text(GTK_LABEL(GTK_BIN(this->button())->child),
			 ::language(*((string*)gtk_object_get_data(GTK_OBJECT(this->widget()),
								   "text"))
				   ).c_str());

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_UPDATE,
		      "XGtkCheckButton::language_update()");
    } // void XGtkCheckButton::language_update()

}
#endif
