/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTK

#include "XGtkLabelEntry.h"

#include "../../../misc/language.h"

namespace GTK_DOKO
{

  /*****************************************************************************
   *
   * XGtkLabelEntry::XGtkLabelEntry(string const& text)
   *
   * Parameters:	text - text for the label
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Constructor
   *
   ****************************************************************************/
  XGtkLabelEntry::XGtkLabelEntry(string const& text) :
    XGtkLabelWidget(text), // the widget
  entry_p(gtk_entry_new()) // the entry
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkLabelEntry::XGtkLabelEntry()");

    gtk_box_pack_end(GTK_BOX(this->container()),
		     this->entry(),
		     TRUE, TRUE, 0);

    // set the text of the label widget
    this->language_update();

    // show all widgets
    gtk_widget_show_all(this->widget());

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkLabelEntry::XGtkLabelEntry()");
  } // XGtkLabelEntry::XGtkLabelEntry()

  /*****************************************************************************
   *
   * XGtkLabelEntry::~XGtkLabelEntry()
   *
   * Parameters:	-
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Destructor
   *
   ****************************************************************************/
  XGtkLabelEntry::~XGtkLabelEntry()
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkLabelEntry::~XGtkLabelEntry()");

    gtk_widget_destroy(this->entry());

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkLabelEntry::~XGtkLabelEntry()");
  } // XGtkLabelEntry::XGtkLabelEntry()

  /*****************************************************************************
   *
   * GtkWidget* XGtkLabelEntry::entry()
   *
   * Parameters:	-
   *
   * Result:	the entry
   *
   * Version:	0.4.3
   *
   * Description:	Result
   *
   ****************************************************************************/
  GtkWidget*
    XGtkLabelEntry::entry()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelEntry::entry()");

      DEBUG_ASSERTION((this->entry_p != NULL),
		      "XGtkLabelEntry::entry()\n"
		      "  `entry_p` == NULL");

      DEBUG_RETURNING(this->entry_p,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelEntry::entry()");
    } // GtkWidget* XGtkLabelEntry::entry()

  /*****************************************************************************
   *
   * string XGtkLabelEntry::entry_text()
   *
   * Parameters:	-
   *
   * Result:	the text of the entry
   *
   * Version:	0.4.3
   *
   * Description:	Result
   *
   ****************************************************************************/
  string
    XGtkLabelEntry::entry_text()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelEntry::entry_text()");

      DEBUG_RETURNING(gtk_entry_get_text(GTK_ENTRY(this->entry())),
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelEntry::entry_text()");
    } // string XGtkLabelEntry::entry_text()

  /*****************************************************************************
   *
   * void XGtkLabelEntry::entry_text_set(string const& string)
   *
   * Parameters:	string - new string for the entry
   *
   * Result:	-
   *
   * Version:	0.4.3
   *
   * Description:	sets the text of the entry
   *
   ****************************************************************************/
  void
    XGtkLabelEntry::entry_text_set(string const& string)
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_UPDATE,
		    "XGtkLabelEntry::entry_text_set()");

      gtk_entry_set_text(GTK_ENTRY(this->entry()),
			 string.c_str());

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_UPDATE,
		      "XGtkLabelEntry::entry_text_set()");
    } // void XGtkLabelEntry::entry_text(string const& string)

}
#endif
