/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTK

#include "XGtkLabelHScale.h"

#include "../../../misc/language.h"

namespace GTK_DOKO {

  /**
   **
   ** constructor
   **
   ** @param	text	text for the label
   **
   ** @return	-
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  XGtkLabelHScale::XGtkLabelHScale(string const& text) :
    XGtkLabelAdjustment(text),
  hscale(gtk_hscale_new(GTK_ADJUSTMENT(gtk_adjustment_new(0, 0, 100,
							  1, 0, 0)
				      ))) // the hscale
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkLabelHScale::XGtkLabelHScale()");

    gtk_scale_set_digits(GTK_SCALE(this->hscale), 0);

    gtk_box_pack_end(GTK_BOX(this->container()),
		     this->hscale,
		     FALSE, FALSE, 0);

    // set the text of the label widget
    this->language_update();

    // show all widgets
    gtk_widget_show_all(this->widget());

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkLabelHScale::XGtkLabelHScale()");
  } // XGtkLabelHScale::XGtkLabelHScale()

  /**
   **
   ** destructor
   **
   ** @param	-
   **
   ** @return	-
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  XGtkLabelHScale::~XGtkLabelHScale()
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkLabelHScale::~XGtkLabelHScale()");

    gtk_widget_destroy(this->hscale);

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkLabelHScale::~XGtkLabelHScale()");
  } // XGtkLabelHScale::XGtkLabelHScale()

  /**
   **
   ** -> result
   **
   ** @param	-
   **
   ** @return	the adjustment
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  GtkAdjustment*
    XGtkLabelHScale::adjustment()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelHScale::adjustment()");

      DEBUG_RETURNING(gtk_range_get_adjustment(GTK_RANGE(this->hscale)),
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelHScale::adjustment()");
    } // GtkAdjustment* XGtkLabelHScale::adjustment()

  /**
   **
   ** -> result
   **
   ** @param	-
   **
   ** @return	the widget with the adjustment
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  GtkWidget*
    XGtkLabelHScale::adjustment_widget()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelHScale::adjustment_widget()");

      DEBUG_RETURNING(this->hscale,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelHScale::adjustment_widget()");
    } // GtkWidget* XGtkLabelHScale::adjustment_widget()

  /**
   **
   ** sets the minimum and the maximum of the adjustment
   **
   ** @param	min	the new minimum
   ** @param	max	the new maximum
   **
   ** @return	-
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  void
    XGtkLabelHScale::minmax_set(int const min, int const max)
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelHScale::minmax_set(min, max)");

      gtk_range_set_adjustment(GTK_RANGE(this->hscale),
			       GTK_ADJUSTMENT(gtk_adjustment_new(this->adjustment()->value,
								 min,
								 max,
								 this->adjustment()->step_increment,
								 this->adjustment()->page_increment,
								 this->adjustment()->page_size
								)
					     )
			      );

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelHScale::minmax_set(min, max)");
    } // GtkWidget* XGtkLabelHScale::minmax_set(unsigned const min, unsigned const max)

  /**
   **
   ** sets the value
   **
   ** @param	value	the new value
   **
   ** @return	-
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  void
    XGtkLabelHScale::value_set(unsigned const value)
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelHScale::value_set(value)");

      gtk_adjustment_set_value(this->adjustment(), value);

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelHScale::value_set(value)");
    } // GtkWidget* XGtkLabelHScale::value_set(unsigned const value)

  /**
   **
   ** -> result
   **
   ** @param	-
   **
   ** @return	the value
   ** 
   ** author	Diether Knof
   **
   ** @version	0.5.2
   **
   **/
  unsigned
    XGtkLabelHScale::value()
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_VALUE,
		    "XGtkLabelHScale::value()");

      int const value = static_cast<int>(this->adjustment()->value);

      DEBUG_RETURNING(((value == -1) ? UINT_MAX : static_cast<unsigned>(value)),
		      INFO_GTK_WIDGET && INFO_GTK_VALUE,
		      "XGtkLabelHScale::value()");
    } // GtkWidget* XGtkLabelHScale::value()

} // namespace GTK_DOKO
#endif // #ifdef USE_UI_GTK
