/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTK

#include "XGtkRadioButton.h"

#include "../../../misc/language.h"

namespace GTK_DOKO
{

  /*****************************************************************************
   *
   * XGtkRadioButton::XGtkRadioButton(string const& text)
   *
   * Parameters:	text - text for the button
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Constructor
   *
   ****************************************************************************/
  XGtkRadioButton::XGtkRadioButton(string const& text) :
    XGtkCheckButton(text)
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkRadioButton::XGtkRadioButton()");

      // remove the button
      delete (string*)gtk_object_get_data(GTK_OBJECT(this->button()), "text");
      gtk_widget_destroy(this->button());

      // create a radio button
      this->button_p = gtk_radio_button_new_with_label(NULL, "");
      gtk_object_set_data(GTK_OBJECT(this->button()),
			  "text",
			  new string(text));

      // set the text of the radio widget
      this->language_update();

      // show all widgets
      gtk_widget_show_all(this->widget());

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_INIT,
		      "XGtkRadioButton::XGtkRadioButton()");
    } // XGtkRadioButton::XGtkRadioButton()


  /*****************************************************************************
   *
   * XGtkRadioButton::XGtkRadioButton(XGtkRadioButton& group, string const& text)
   *
   * Parameters:	group - a radio button, which is in the same group
   *		text - text for the button
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Constructor
   *
   ****************************************************************************/
  XGtkRadioButton::XGtkRadioButton(XGtkRadioButton& group, string const& text) :
    XGtkCheckButton(text)
    {
      DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkRadioButton::XGtkRadioButton()");

      // remove the button
      delete (string*)gtk_object_get_data(GTK_OBJECT(this->button()), "text");
      gtk_widget_destroy(this->button());

      // create a radio button
      this->button_p = gtk_radio_button_new_with_label_from_widget(GTK_RADIO_BUTTON(group.button()),
								   "");
      gtk_object_set_data(GTK_OBJECT(this->button()),
			  "text",
			  new string(text));

      // set the text of the radio widget
      this->language_update();

      // show all widgets
      gtk_widget_show_all(this->widget());

      DEBUG_RETURNING(VOID,
		      INFO_GTK_WIDGET && INFO_GTK_INIT,
		      "XGtkRadioButton::XGtkRadioButton()");
    } // XGtkRadioButton::XGtkRadioButton()

  /*****************************************************************************
   *
   * XGtkRadioButton::~XGtkRadioButton()
   *
   * Parameters:	-
   *
   * Result:	-
   * 
   * Version:	0.4.3
   *
   * Description:	Destructor
   *
   ****************************************************************************/
  XGtkRadioButton::~XGtkRadioButton()
  {
    DEBUG_CALLING(INFO_GTK_WIDGET && INFO_GTK_INIT,
		  "XGtkRadioButton::~XGtkRadioButton()");

    DEBUG_RETURNING(VOID,
		    INFO_GTK_WIDGET && INFO_GTK_INIT,
		    "XGtkRadioButton::~XGtkRadioButton()");
  } // XGtkRadioButton::XGtkRadioButton()

}
#endif
