/**********************************************************************
 *
 *   FreeDoko a Doppelkopf-Game
 *    
 *   Copyright (C) 2001-2004  by Diether Knof and Borg Enders
 *
 *   This program is free software; you can redistribute it and/or 
 *   modify it under the terms of the GNU General Public License as 
 *   published by the Free Software Foundation; either version 2 of 
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details. 
 *   You can find this license in the file 'gpl.txt'.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, 
 *   MA  02111-1307  USA
 *
 *  Contact:
 *    Diether Knof dknof@gmx.de
 *    Borg Enders  borg@borgsoft.de
 *
 *********************************************************************/

#include "constants.h"

#ifdef USE_UI_GTKMM

#include "ui.h"
#include "main_window.h"
#include "menu.h"
#include "table.h"
#include "trick.h"

#include "../../card/trick.h"

namespace UI_GTKMM_NS {

  /**
   **
   ** a new trick is opened
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @version	0.6.3
   **
   ** @author	Diether Knof
   **
   **/
  void
    UI_GTKMM::trick_open()
    {
      DEBUG_CALLING(INFO_GTKMM_UI,
		    "UI_GTKMM::trick_open()");

      this->main_window->menu->trick_open();
      this->table->trick().draw(true);

      DEBUG_RETURNING(VOID,
		      INFO_GTKMM_UI,
		      "UI_GTKMM::trick_open()");
    } // void UI_GTKMM::trick_open()

  /**
   **
   ** the trick is full.
   ** show the 'full trick'-window or wait
   **
   ** @param	trick	the trick
   **
   ** @return	-
   **
   ** @version	0.6.1
   **
   ** @author	Diether Knof
   **
   **/
  void
    UI_GTKMM::trick_full(::Trick const& trick)
    {
      DEBUG_CALLING(INFO_GTKMM_UI,
		    "UI_GTKMM::trick_full(trick)");

      this->table->trick_full(trick);

      DEBUG_RETURNING(VOID,
		      INFO_GTKMM_UI,
		      "UI_GTKMM::trick_full(trick)");
    } // void UI_GTKMM::trick_full(trick)

  /**
   **
   ** move the trick in the pile of the winnerplayer
   **
   ** @param	trick	the trick
   **
   ** @return	-
   **
   ** @version	0.5.2
   **
   ** @author	Diether Knof
   **
   ** @todo	all
   **
   **/
  void
    UI_GTKMM::trick_move_in_pile(::Trick const& trick)
    {
      DEBUG_CALLING(INFO_GTKMM_UI,
		    "UI_GTKMM::trick_move_in_pile(trick)");

      this->table->trick_move_in_pile(trick);

      DEBUG_RETURNING(VOID,
		      INFO_GTKMM_UI,
		      "UI_GTKMM::trick_move_in_pile(trick)");
    } // void UI_GTKMM::trick_move_in_pile(trick)

  /**
   **
   ** the trick is closed
   **
   ** @param	-
   **
   ** @return	-
   **
   ** @version	0.5.2
   **
   ** @author	Diether Knof
   **
   ** @todo	all
   **
   **/
  void
    UI_GTKMM::trick_close()
    {
      DEBUG_CALLING(INFO_GTKMM_UI,
		    "UI_GTKMM::trick_close()");

      DEBUG_RETURNING(VOID,
		      INFO_GTKMM_UI,
		      "UI_GTKMM::trick_close()");
    } // void UI_GTKMM::trick_close()

} // namespace UI_GTKMM_NS

#endif // #ifdef USE_UI_GTKMM
